import os
import requests

from collections import namedtuple
from typing import List, Dict
from pathlib import Path
from rich.console import Console
from rich.table import Table
from rich.progress import track
from rich.padding import Padding

Row = namedtuple('Row', ['name', 'current_version', 'latest_version'])

console = Console()


class UpToDate:
    DEFAULT_FILE_PATH = Path('requirements.txt')

    def __init__(self) -> None:
        self.scanner = RequirementsScanner()
        self.pypi = PyPI()

    def run(self, file_paths: List[Path]):
        if not file_paths:
            if os.path.isfile(self.DEFAULT_FILE_PATH):
                self._check_file(self.DEFAULT_FILE_PATH)
            else:
                console.print('No requirements file found', style='yellow')
        else:
            self._check_files(file_paths)

    def _check_files(self, file_paths: List[Path]) -> None:
        for file_path in file_paths:
            self._check_file(file_path)

    def _check_file(self, file_path: Path) -> None:
        console.print(file_path, style='blue')
        dependences = self.scanner.get_dependences(file_path)
        versions = self.pypi.get_latest_versions(list(dependences.keys()))
        rows = self._prepare_rows(dependences, versions)
        if len(rows) == 0:
            console.print(
                Padding('Everything is up to date', (0, 0, 1, 2)), style='green'
            )
        else:
            self._draw_table(rows)

    def _prepare_rows(
        self, dependences: Dict[str, str], latest_versions: Dict[str, str]
    ) -> List[Row]:
        return [
            Row(
                name=name,
                current_version=current_version,
                latest_version=latest_versions[name],
            )
            for name, current_version in dependences.items()
            if current_version != latest_versions[name]
        ]

    def _draw_table(self, rows: List[Row]) -> None:
        table = Table(show_header=True, header_style='bold cyan')
        table.add_column('Name', style='dim', justify='left')
        table.add_column('Current', justify='center')
        table.add_column('Latest', justify='center')

        for row in rows:
            table.add_row(row.name, row.current_version, row.latest_version)

        console.print(Padding(table, (0, 0, 1, 2)))


class RequirementsScanner:
    def get_dependences(self, file_path: Path) -> Dict[str, str]:
        dependences = {}

        with open(file_path, 'r') as f:
            for line in f.readlines():
                if not line.strip():
                    continue
                if not self._is_supported(line):
                    console.print(
                        Padding('SKIPPED: {}'.format(line), (0, 0, 0, 2)),
                        style='yellow',
                    )
                    continue

                name, version = line.strip().split('==')
                dependences[name] = version

        return dependences

    def _is_supported(self, line: str) -> bool:
        if '==' not in line:
            return False
        if '[' in line and ']' in line:
            return False

        return True


class PyPI:
    URL_PATTERN = 'https://pypi.org/pypi/{}/json'
    PROGRESS_BAR_THRESHOLD = 10

    def get_latest_versions(self, dependences: List[str]) -> Dict[str, str]:
        versions = {}

        for dependence in self._warp(dependences):
            url = self.URL_PATTERN.format(dependence)
            response = requests.get(url)
            data = response.json()
            latest_version = data['info']['version']
            versions[dependence] = latest_version

        return versions

    def _warp(self, dependences: List[str]):
        if len(dependences) >= self.PROGRESS_BAR_THRESHOLD:
            return track(dependences, description='Checking...')

        return dependences



import pytest

from rich.console import Console
from uptodate.core import UpToDate, Row


@pytest.fixture
def uptodate():
    return UpToDate()


@pytest.fixture
def dependences():
    return {'click': '6.7', 'pytest': '3.0.0'}


@pytest.fixture
def latest_versions():
    return {'click': '6.7', 'pytest': '3.2.5'}


@pytest.fixture
def rows():
    return [Row(name='pytest', current_version='3.0.0', latest_version='3.2.5')]


def test_prepare_rows(uptodate, dependences, latest_versions, rows):
    prepared_rows = uptodate._prepare_rows(dependences, latest_versions)

    assert prepared_rows == rows



import pytest

from unittest.mock import patch, mock_open
from uptodate.core import RequirementsScanner


@pytest.fixture
def scanner():
    return RequirementsScanner()


@pytest.fixture
def open_mock():
    read_data = '\n'.join(['click==6.7', 'pytest==3.2.5', 'celery[redis]==4.0.2'])

    return mock_open(read_data=read_data)


def test_get_dependences(scanner, open_mock):
    file_path = 'requirements.txt'
    with patch("builtins.open", open_mock) as mock_file:
        dependences = scanner.get_dependences(file_path)

        mock_file.assert_called_once_with(file_path, 'r')

    assert dependences == {'click': '6.7', 'pytest': '3.2.5'}


def test_is_supported(scanner):
    assert scanner._is_supported('click==6.7') is True
    assert scanner._is_supported('click.zip') is False
    assert scanner._is_supported('celery[redis]') is False



import typer

from pathlib import Path
from typing import List

try:
    from uptodate.core import UpToDate
except Exception:
    from core import UpToDate  # type: ignore


def up_to_date(file_paths: List[Path] = typer.Argument(None, exists=True)):
    """Scan requirements.txt file for dependences which are not up to date"""

    uptodate = UpToDate()
    uptodate.run(file_paths)


if __name__ == '__main__':
    typer.run(up_to_date)



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['uptodate', 'uptodate.tests']

package_data = \
{'': ['*'],
 'uptodate': ['.cache/*',
              '.cache/v/*',
              '.cache/v/cache/*',
              '.mypy_cache/*',
              '.mypy_cache/3.6/*',
              '.mypy_cache/3.6/click/*',
              '.mypy_cache/3.6/collections/*',
              '.mypy_cache/3.6/distutils/*',
              '.mypy_cache/3.6/tests/*']}

install_requires = \
['typer==0.2.1', 'requests==2.23.0', 'rich==2.0.0']

entry_points = \
{'console_scripts': ['uptodate = uptodate.cli']}

setup(name='uptodate',
      version='0.3.1',
      description='Command line script checking if your requirements are uptodate',
      author='Paweł Jastrzębski',
      author_email='pj@napcode.eu',
      url='https://gitlab.com/havk/uptodate',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
     )
